@@ -83,6 +83,7 @@ const bool havegl=false;
 mode_t mask;
   
 string systemDir=ASYMPTOTE_SYSDIR;
+string defaultEPSdriver="eps2write";
 
 #ifndef __MSDOS__
   
@@ -90,14 +91,15 @@ bool msdos=false;
 string HOME="HOME";
 string docdir=ASYMPTOTE_DOCDIR;
 const char pathSeparator=':';
-string defaultPSViewer="gv";
 #ifdef __APPLE__
+string defaultPSViewer="open";
 string defaultPDFViewer="open";
 #else  
+string defaultPSViewer="gv";
 string defaultPDFViewer="acroread";
 #endif  
 string defaultGhostscript="gs";
-string defaultGhostscriptLibrary="/usr/lib/libgs.so";
+string defaultGhostscriptLibrary="";
 string defaultDisplay="display";
 string defaultAnimate="animate";
 void queryRegistry() {}
@@ -183,7 +185,7 @@ string getEntry(const string& key)
 
 void queryRegistry()
 {
-  defaultGhostscriptLibrary=getEntry("GPL Ghostscript/*/GS_DLL");
+  string defaultGhostscriptLibrary=getEntry("GPL Ghostscript/*/GS_DLL");
   if(defaultGhostscriptLibrary.empty())
     defaultGhostscriptLibrary=getEntry("AFPL Ghostscript/*/GS_DLL");
   
@@ -707,7 +709,8 @@ struct engineSetting : public argumentSetting {
     string str=optarg;
     
     if(str == "latex" || str == "pdflatex" || str == "xelatex" ||
-       str == "tex" || str == "pdftex" || str == "context" || str == "none") {
+       str == "tex" || str == "pdftex" || str == "luatex" ||
+       str == "lualatex" || str == "context" || str == "none") {
       value=str;
       return true;
     }
@@ -866,7 +869,7 @@ void addOption(option *o) {
 
 void version()
 {
-  cerr << PROGRAM << " version " << VERSION << SVN_REVISION
+  cerr << PROGRAM << " version " << REVISION
        << " [(C) 2004 Andy Hammerlindl, John C. Bowman, Tom Prince]" 
        << endl;
 }
@@ -1065,14 +1068,14 @@ void initSettings() {
 // ALT LEFT: pan
   const char *leftbutton[]={"rotate","zoom","shift","pan",NULL};
   
-// MIDDLE: menu (must be unmodified; ignores Shift, Ctrl, and Alt)
-  const char *middlebutton[]={"menu",NULL};
+// MIDDLE:
+  const char *middlebutton[]={NULL};
   
-// RIGHT: zoom/menu (must be unmodified)
+// RIGHT: zoom
 // SHIFT RIGHT: rotateX
 // CTRL RIGHT: rotateY
 // ALT RIGHT: rotateZ
-  const char *rightbutton[]={"zoom/menu","rotateX","rotateY","rotateZ",NULL};
+  const char *rightbutton[]={"zoom","rotateX","rotateY","rotateZ",NULL};
   
 // WHEEL_UP: zoomin
   const char *wheelup[]={"zoomin",NULL};
@@ -1109,6 +1112,10 @@ void initSettings() {
                             "Show 3D toolbar in PDF output", true));
   addOption(new boolSetting("axes3", 0,
                             "Show 3D axes in PDF output", true));
+  addOption(new boolSetting("envmap", 0,
+                            "Enable environment map image-based lighting (Experimental)", false));
+                            
+                            
   addOption(new realSetting("render", 0, "n",
                             "Render 3D graphics using n pixels per bp (-1=auto)",
                             havegl ? -1.0 : 0.0));
@@ -1146,6 +1153,8 @@ void initSettings() {
   addOption(new versionOption("version", 0, "Show version"));
 
   addOption(new pairSetting("offset", 'O', "pair", "PostScript offset"));
+  addOption(new pairSetting("aligndir", 0, "pair",
+                             "Directional page alignment (overrides align)"));
   addOption(new alignSetting("align", 'a', "C|B|T|Z",
                              "Center, Bottom, Top, or Zero page alignment",
                              "C"));
@@ -1161,7 +1170,7 @@ void initSettings() {
   addOption(new boolSetting("keepaux", 0,
                             "Keep intermediate LaTeX .aux files"));
   addOption(new engineSetting("tex", 0, "engine",
-                              "latex|pdflatex|xelatex|tex|pdftex|context|none",
+                              "latex|pdflatex|xelatex|lualatex|tex|pdftex|luatex|context|none",
                               "latex"));
 
   addOption(new boolSetting("twice", 0,
@@ -1227,6 +1236,8 @@ void initSettings() {
                               ".."));
   addOption(new boolSetting("multiline", 0,
                             "Input code over multiple lines at the prompt"));
+  addOption(new boolSetting("xasy", 0,
+                            "Special interactive mode for xasy"));
 
   addOption(new boolSetting("wait", 0,
                             "Wait for child processes to finish before exiting"));
@@ -1236,7 +1247,7 @@ void initSettings() {
                             true));
                             
   addOption(new boolSetting("quiet", 'q',
-                            "Suppress welcome message"));
+                            "Suppress welcome text and noninteractive stdout"));
   addOption(new boolSetting("localhistory", 0,
                             "Use a local interactive history file"));
   addOption(new IntSetting("historylines", 0, "n",
@@ -1263,6 +1274,7 @@ void initSettings() {
   addOption(new userSetting("user", 'u', "string",
                             "General purpose user string"));
   
+  addOption(new IntSetting("maxvertices", 0, "n", "Maximum number of vertices to queue", 0));
   addOption(new realSetting("zoomfactor", 0, "factor", "Zoom step factor",
                             1.05));
   addOption(new realSetting("zoomstep", 0, "step", "Mouse motion zoom step",
@@ -1298,6 +1310,7 @@ void initSettings() {
   addOption(new envSetting("psviewer", defaultPSViewer));
   addOption(new envSetting("gs", defaultGhostscript));
   addOption(new envSetting("libgs", defaultGhostscriptLibrary));
+  addOption(new envSetting("epsdriver", defaultEPSdriver));
   addOption(new envSetting("texpath", ""));
   addOption(new envSetting("texcommand", ""));
   addOption(new envSetting("dvips", "dvips"));
@@ -1321,17 +1334,26 @@ void initSettings() {
 int numArgs() { return argCount; }
 char *getArg(int n) { return argList[n]; }
 
-void setInteractive() {
+void setInteractive()
+{
   if(numArgs() == 0 && !getSetting<bool>("listvariables") && 
      getSetting<string>("command").empty() &&
-     (isatty(STDIN_FILENO) || getSetting<Int>("inpipe") >= 0))
+     (isatty(STDIN_FILENO) || getSetting<Int>("xasy")))
     interact::interactive=true;
   
-  historyname=getSetting<bool>("localhistory") ? 
-    (string(getPath())+dirsep+"."+suffix+"_history") : (initdir+"/history");
+  if(getSetting<bool>("localhistory"))
+    historyname=string(getPath())+dirsep+"."+suffix+"_history";
+  else {
+    if(mkdir(initdir.c_str(),0777) != 0 && errno != EEXIST)
+      cerr << "failed to create directory "+initdir+"." << endl;
+    historyname=initdir+"/history";
+  }
+  if(verbose > 1)
+     cerr << "Using history " << historyname << endl;
 }
 
-bool view() {
+bool view()
+{
   if (interact::interactive)
     return getSetting<bool>("interactiveView");
   else
@@ -1339,7 +1361,8 @@ bool view() {
       (numArgs() == 1 || getSetting<bool>("multipleView"));
 }
 
-bool trap() {
+bool trap()
+{
   if (interact::interactive)
     return !getSetting<bool>("interactiveMask");
   else
@@ -1358,7 +1381,11 @@ string lookup(const string& symbol)
 {
   string s;
   mem::vector<string> cmd;
-  cmd.push_back("kpsewhich");
+  string kpsewhich="kpsewhich";
+  string fullname=stripFile(argv0)+kpsewhich;
+  std::ifstream exists(fullname.c_str());
+  if(!exists) fullname=kpsewhich;
+  cmd.push_back(fullname);
   cmd.push_back("--var-value="+symbol);
   iopipestream pipe(cmd);
   pipe >> s;
@@ -1395,9 +1422,10 @@ void initDir() {
   if(mask == 0) mask=0027;
   umask(mask);
 #endif  
-  if(verbose > 1)
-    cerr << "Using configuration directory " << initdir << endl;
-  mkdir(initdir.c_str(),0777);
+  if(access(initdir.c_str(),F_OK) == 0) {
+    if(verbose > 1)
+      cerr << "Using configuration directory " << initdir << endl;
+  }
 }
   
 void setPath() {
@@ -1412,7 +1440,8 @@ void setPath() {
     }
     if(i < asydir.length()) searchPath.push_back(asydir.substr(i));
   }
-  searchPath.push_back(initdir);
+  if(access(initdir.c_str(),F_OK) == 0)
+    searchPath.push_back(initdir);
   string sysdir=getSetting<string>("sysdir");
   if(sysdir != "")
     searchPath.push_back(sysdir);
@@ -1440,51 +1469,47 @@ void SetPageDimensions() {
   }
 }
 
-bool xe(const string& texengine) {
+bool xe(const string& texengine)
+{
   return texengine == "xelatex";
 }
 
-bool context(const string& texengine) {
+bool lua(const string& texengine)
+{
+  return texengine == "luatex" || texengine == "lualatex";
+}
+
+bool context(const string& texengine)
+{
   return texengine == "context";
 }
 
-bool pdf(const string& texengine) {
+bool pdf(const string& texengine)
+{
   return texengine == "pdflatex" || texengine == "pdftex" || xe(texengine) ||
-    context(texengine);
+    lua(texengine) || context(texengine);
 }
 
-bool latex(const string& texengine) {
+bool latex(const string& texengine)
+{
   return texengine == "latex" || texengine == "pdflatex" || 
-    texengine == "xelatex";
+    texengine == "xelatex" || texengine == "lualatex";
 }
 
-string nativeformat() {
+string nativeformat()
+{
   return pdf(getSetting<string>("tex")) ? "pdf" : "eps";
 }
 
-string defaultformat() {
+string defaultformat()
+{
   string format=getSetting<string>("outformat");
   return (format.empty()) ? nativeformat() : format;
 }
 
-// Begin TeX put command.
-const char *beginput(const string& texengine) {
-  if(context(texengine))
-    return "\\put";
-  else
-    return "\\put(#1,#2)";
-}
-
-// End TeX put command.
-const char *endput(const string& texengine) {
-  if(context(texengine)) 
-    return " at #1 #2"; 
-  else
-    return "";
-}
-
 // TeX special command to set up currentmatrix for typesetting labels.
-const char *beginlabel(const string& texengine) {
+const char *beginlabel(const string& texengine)
+{
   if(pdf(texengine))
     return xe(texengine) ? "\\special{pdf:literal q #5 0 0 cm}" :
       "\\special{pdf:q #5 0 0 cm}";
@@ -1494,7 +1519,8 @@ const char *beginlabel(const string& texengine) {
 }
 
 // TeX special command to restore currentmatrix after typesetting labels.
-const char *endlabel(const string& texengine) {
+const char *endlabel(const string& texengine)
+{
   if(pdf(texengine))
     return xe(texengine) ? "\\special{pdf:literal Q}" : "\\special{pdf:Q}";
   else
@@ -1502,7 +1528,8 @@ const char *endlabel(const string& texengine) {
 }
 
 // TeX macro to typeset raw postscript code
-const char *rawpostscript(const string& texengine) {
+const char *rawpostscript(const string& texengine)
+{
   if(pdf(texengine))
     return "\\def\\ASYraw#1{#1}";
   else
@@ -1514,7 +1541,8 @@ const char *rawpostscript(const string& texengine) {
 }
 
 // TeX macro to begin picture
-const char *beginpicture(const string& texengine) {
+const char *beginpicture(const string& texengine) 
+{
   if(latex(texengine))
     return "\\begin{picture}";
   if(context(texengine))
@@ -1524,7 +1552,8 @@ const char *beginpicture(const string& texengine) {
 }
 
 // TeX macro to end picture
-const char *endpicture(const string& texengine) {
+const char *endpicture(const string& texengine)
+{
   if(latex(texengine))
     return "\\end{picture}%";
   else if(context(texengine))
@@ -1534,7 +1563,8 @@ const char *endpicture(const string& texengine) {
 }
 
 // Begin TeX special command.
-const char *beginspecial(const string& texengine) {
+const char *beginspecial(const string& texengine)
+{
   if(pdf(texengine))
     return xe(texengine) ? "\\special{pdf:literal " : "\\special{pdf:";
   else
@@ -1542,18 +1572,9 @@ const char *beginspecial(const string& texengine) {
 }
 
 // End TeX special command.
-const char *endspecial() {
-  return "}%";
-}
-
-bool fataltex[]={false,true};
-const char *pdftexerrors[]={"! "," ==> Fatal error",NULL};
-const char *texerrors[]={"! ",NULL};
-
-// Messages that signify a TeX error.
-const char **texabort(const string& texengine)
+const char *endspecial()
 {
-  return settings::pdf(texengine) ? pdftexerrors : texerrors;
+  return "}%";
 }
 
 string texcommand()
@@ -1572,23 +1593,23 @@ string texprogram()
 Int getScroll() 
 {
   Int scroll=settings::getSetting<Int>("scroll");
-#ifdef HAVE_LIBCURSES  
   if(scroll < 0) {
+#ifdef HAVE_LIBCURSES  
     static char *terminal=NULL;
     if(!terminal)
       terminal=getenv("TERM");
     if(terminal) {
-      int error;
-      error=setupterm(terminal,1,&error);
 #ifndef __MSDOS__      
+      int error=setupterm(terminal,1,&error);
       if(error == 0) scroll=lines > 2 ? lines-1 : 1;
       else
 #endif
         scroll=0;
     } else scroll=0;
-
-  }
+#else
+    scroll=0;
 #endif
+  }
   return scroll;
 }
 
